<?php

ini_set('max_execution_time', 300); //300 seconds = 5 minutes

require __DIR__ . '/vendor/autoload.php';
require_once('includes/_dbconfig.php');
require_once('includes/PasswordHash.php');

date_default_timezone_set('Africa/Nairobi');

$app = new \Slim\App();

$app->get('/pwop/gifts/', 'getGifts');
$app->get('/pwop/gifts/{id}', 'getGift');
$app->post('/pwop/gifts/', 'addGift');
$app->get('/pwop/gifts/drinks/', 'getBottleNumber');
$app->get('/pwop/gifts/notebooks/', 'getNoteBookNumber');

$app->get('/pwop/scores/', 'getScores');
$app->get('/pwop/scores/{id}', 'getScore');
$app->post('/pwop/scores/', 'addScore');

$app->get('/pwop/draws/', 'getDraws');
$app->get('/pwop/draws/{id}', 'getDraw');
$app->post('/pwop/draws/', 'addDraw');

$app->get('/pwop/users/', 'getUsers');
$app->get('/pwop/users/{id}', 'getUser');
$app->get('/pwop/users/phone/{phone}', 'getUserByPhone');
$app->post('/pwop/users/', 'addUser');

$app->get('/pwop/password/{password}', 'getPassword'); //Get a hash password

$app->get('/pwop/allgifts/', 'getAllGifts');
$app->get('/pwop/allscores/', 'getAllScores');

$app->run();




/**
 * Function Name: getBottleNumber
 *
 * Retrieves total number of bottles awarded today 
 * 
 * @return (object) ($token) the actual bottles number
 */
function getBottleNumber($request, $response, $args) {	

	$sql = "SELECT count(*) as total FROM gifts WHERE gift =:gift AND DATE(created_at) = DATE(NOW())";
	
	$drink = 'drink';

	try {
		
		$db = getConnection();
		$stmt = $db->prepare($sql);	
		$stmt->bindParam("gift", $drink);	
		$stmt->execute();				  
		$number = $stmt->fetchObject();	 
		$db = null;
		
		$data = array('success' => true, 'number' => $number);
		return $response->withJson($data,200);
		
	} catch(PDOException $e) {		
		
		$data = array('success' => false, 
					  'message' => $e->getMessage(), 
					  'code' => $e->getCode(),
					  'one_line' => $e->getLine());

		return $response->withJson($data,200);

	}
}	







/**
 * Function Name: getNoteBookNumber
 *
 * Retrieves total number of notebooks awarded today 
 * 
 * @return (object) ($token) the actual notebooks number
 */
function getNoteBookNumber($request, $response, $args) {	

	$sql = "SELECT count(*) as total FROM gifts WHERE gift =:gift AND DATE(created_at) = DATE(NOW())";
	
	$drink = 'notebook';

	try {
		
		$db = getConnection();
		$stmt = $db->prepare($sql);	
		$stmt->bindParam("gift", $drink);	
		$stmt->execute();				  
		$number = $stmt->fetchObject();	 
		$db = null;
		
		$data = array('success' => true, 'number' => $number);
		return $response->withJson($data,200);
		
	} catch(PDOException $e) {		
		
		$data = array('success' => false, 
					  'message' => $e->getMessage(), 
					  'code' => $e->getCode(),
					  'one_line' => $e->getLine());

		return $response->withJson($data,200);

	}
}	




/**
 * Function Name: getPassword
 *
 * Get password
 *
 * @param (array) ([staff details]) the staff details
 * @return (number) ($rows) the number of rows added
 *
 */
function getPassword($request, $response, $args) {
	
	$password = $args['password'];
	$hasher = new PasswordHash(8, FALSE);
	$pass_word = $hasher->HashPassword($password);

	if($pass_word != false){
		$data = array('success' => true, 'password' => $pass_word);
		return $response->withJson($data,200);
	}else{
		$data = array('success' => false, 'error' => 'Sorry unable to generate password!');
		return $response->withJson($data,200);
	}		   
	
}




/**
 * Function Name: getUsers
 *
 * Retrieves all users 
 * 
 * @return (object) ($token) the actual users details 
 */
function getUsers($request, $response, $args) {	

	$sql = "SELECT id,name,mobile_number,department FROM users";

	try {
		$db = getConnection();
		$stmt = $db->prepare($sql);	
		$stmt->execute();				  
		$users = $stmt->fetchAll(PDO::FETCH_OBJ); 	 
		$db = null;
		
		$data = array('success' => true, 'users' => $users);
		return $response->withJson($data,200);
		
	} catch(PDOException $e) {		
		
		$data = array('success' => false, 
					  'message' => $e->getMessage(), 
					  'code' => $e->getCode(),
					  'one_line' => $e->getLine());

		return $response->withJson($data,200);

	}
}	




/**
 * Function Name: getUser
 *
 * Retrieves a user by id
 * 
 * @return (object) ($token) the actual user details 
 */
function getUser($request, $response, $args) {	

	$id = $args['id'];

	$sql = "SELECT id,name,mobile_number,department FROM users WHERE id =:id";

	try {
		$db = getConnection();
		$stmt = $db->prepare($sql);	
		$stmt->bindParam("id", $id);	
		$stmt->execute();				  
		$user = $stmt->fetchObject();	 
		$db = null;
		
		$data = array('success' => true, 'user' => $user);
		return $response->withJson($data,200);
		
	} catch(PDOException $e) {		
		
		$data = array('success' => false, 
					  'message' => $e->getMessage(), 
					  'code' => $e->getCode(),
					  'one_line' => $e->getLine());

		return $response->withJson($data,200);

	}
}	







/**
 * Function Name: getUserByPhone
 *
 * Retrieves a user by phone
 * 
 * @return (object) ($token) the actual user details 
 */
function getUserByPhone($request, $response, $args) {	

	$phone = $args['phone'];

	$sql = "SELECT id,name,mobile_number,department FROM users WHERE mobile_number =:phone";

	try {
		$db = getConnection();
		$stmt = $db->prepare($sql);	
		$stmt->bindParam("phone", $phone);	
		$stmt->execute();				  
		$user = $stmt->fetchObject();	 
		$db = null;
		
		$data = array('success' => true, 'user' => $user);
		return $response->withJson($data,200);
		
	} catch(PDOException $e) {		
		
		$data = array('success' => false, 
					  'message' => $e->getMessage(), 
					  'code' => $e->getCode(),
					  'one_line' => $e->getLine());

		return $response->withJson($data,200);

	}
}




/**
 * Function Name: addUser
 *
 * Adds a new user
 *
 * @param (array) ([user details]) the user details
 * @return (number) ($rows) the number of rows added
 *
 */
function addUser($request, $response, $args) {	
	
	$user = json_decode($request->getBody());
	
	$sql = "INSERT INTO users (name,mobile_number,department) VALUES (:name,:mobile_number,:department)";
	    
	    try {
	    	
	    	$name = cleanField($user->name);
	    	$mobile_number = cleanField($user->mobile_number);
	    	$department = cleanField($user->department);
	    		    			    	
	    	$db = getConnection();
			$stmt = $db->prepare($sql);
			
			$stmt->bindParam("name", $name);
			$stmt->bindParam("mobile_number", $mobile_number);
			$stmt->bindParam("department", $department);		
			
			$rows = $stmt->execute();					
			$lastId = $db->lastInsertId();

			$db = null;

			if($rows > 0){
							
				$data = array('success' => true, 'id' => $lastId);
				return $response->withJson($data,200);					    
			    
			}else{
		    	
		    	$data = array('success' => false, 'message' => 'Sorry unable to add user');
				return $response->withJson($data,200);
		    }	    
		    
	    
	    } catch(PDOException $e) {		
		     
		    $data = array('success' => false, 'message' => $e->getMessage());
			return $response->withJson($data,200);
	    }		   
	
}






/**
 * Function Name: getDraws
 *
 * Retrieves all draws 
 * 
 * @return (object) ($token) the actual draws details 
 */
function getDraws($request, $response, $args) {	

	$sql = "SELECT id,user_id,score FROM draws";

	try {
		$db = getConnection();
		$stmt = $db->prepare($sql);	
		$stmt->execute();				  
		$draws = $stmt->fetchAll(PDO::FETCH_OBJ); 	 
		$db = null;
		
		$data = array('success' => true, 'draws' => $draws);
		return $response->withJson($data,200);
		
	} catch(PDOException $e) {		
		
		$data = array('success' => false, 
					  'message' => $e->getMessage(), 
					  'code' => $e->getCode(),
					  'one_line' => $e->getLine());

		return $response->withJson($data,200);

	}
}	




/**
 * Function Name: getDraw
 *
 * Retrieves a draw by id
 * 
 * @return (object) ($token) the actual draw details 
 */
function getDraw($request, $response, $args) {	

	$id = $args['id'];

	$sql = "SELECT id,user_id,score FROM draws WHERE id =:id";

	try {
		$db = getConnection();
		$stmt = $db->prepare($sql);	
		$stmt->bindParam("id", $id);	
		$stmt->execute();				  
		$draw = $stmt->fetchObject();	 
		$db = null;
		
		$data = array('success' => true, 'draw' => $draw);
		return $response->withJson($data,200);
		
	} catch(PDOException $e) {		
		
		$data = array('success' => false, 
					  'message' => $e->getMessage(), 
					  'code' => $e->getCode(),
					  'one_line' => $e->getLine());

		return $response->withJson($data,200);

	}
}	




/**
 * Function Name: addDraw
 *
 * Adds a new draw
 *
 * @param (array) ([draw details]) the draw details
 * @return (number) ($rows) the number of rows added
 *
 */
function addDraw($request, $response, $args) {	
	
	$draw = json_decode($request->getBody());
	
	$sql = "INSERT INTO draws (user_id,score) VALUES (:user_id, :score)";
	    
	    try {
	    	
	    	$score = cleanField($draw->score);
	    	$user_id = cleanField($draw->user_id);
	    		    			    	
	    	$db = getConnection();
			$stmt = $db->prepare($sql);
			
			$stmt->bindParam("score", $score);
			$stmt->bindParam("user_id", $user_id);		
			
			$rows = $stmt->execute();					

			$db = null;

			if($rows > 0){

				$message = 'Draw added successfully';
				$data = array('success' => true, 'message' => $message);
				return $response->withJson($data,200);					    
			    
			}else{
		    	
		    	$data = array('success' => false, 'message' => 'Sorry unable to add draw');
				return $response->withJson($data,200);
		    }	    
		    
	    
	    } catch(PDOException $e) {		
		     
		    $data = array('success' => false, 'message' => $e->getMessage());
			return $response->withJson($data,200);
	    }		   
	
}






/**
 * Function Name: getGifts
 *
 * Retrieves all gifts 
 * 
 * @return (object) ($token) the actual gifts details 
 */
function getGifts($request, $response, $args) {	

	$sql = "SELECT id,user_id,gift FROM gifts";

	try {
		$db = getConnection();
		$stmt = $db->prepare($sql);	
		$stmt->execute();				  
		$gifts = $stmt->fetchAll(PDO::FETCH_OBJ); 	 
		$db = null;
		
		$data = array('success' => true, 'gifts' => $gifts);
		return $response->withJson($data,200);
		
	} catch(PDOException $e) {		
		
		$data = array('success' => false, 
					  'message' => $e->getMessage(), 
					  'code' => $e->getCode(),
					  'one_line' => $e->getLine());

		return $response->withJson($data,200);

	}
}	




/**
 * Function Name: getGift
 *
 * Retrieves a gift by id
 * 
 * @return (object) ($token) the actual gift details 
 */
function getGift($request, $response, $args) {	

	$id = $args['id'];

	$sql = "SELECT id,user_id,gift FROM gifts WHERE id =:id";

	try {
		$db = getConnection();
		$stmt = $db->prepare($sql);	
		$stmt->bindParam("id", $id);	
		$stmt->execute();				  
		$gift = $stmt->fetchObject();	 
		$db = null;
		
		$data = array('success' => true, 'gift' => $gift);
		return $response->withJson($data,200);
		
	} catch(PDOException $e) {		
		
		$data = array('success' => false, 
					  'message' => $e->getMessage(), 
					  'code' => $e->getCode(),
					  'one_line' => $e->getLine());

		return $response->withJson($data,200);

	}
}	




/**
 * Function Name: addGift
 *
 * Adds a new gift
 *
 * @param (array) ([gift details]) the gift details
 * @return (number) ($rows) the number of rows added
 *
 */
function addGift($request, $response, $args) {	
	
	$gift = json_decode($request->getBody());
	
	$sql = "INSERT INTO gifts (user_id,gift) VALUES (:user_id, :gift)";
	    
	    try {
	    	
	    	$gift = cleanField($gift->gift);
	    	$user_id = cleanField($gift->user_id);
	    		    			    	
	    	$db = getConnection();
			$stmt = $db->prepare($sql);
			
			$stmt->bindParam("gift", $gift);
			$stmt->bindParam("user_id", $user_id);		
			
			$rows = $stmt->execute();					

			$db = null;

			if($rows > 0){

				$message = 'Gift added successfully';
				$data = array('success' => true, 'message' => $message);
				return $response->withJson($data,200);					    
			    
			}else{
		    	
		    	$data = array('success' => false, 'message' => 'Sorry unable to add gift');
				return $response->withJson($data,200);
		    }	    
		    
	    
	    } catch(PDOException $e) {		
		     
		    $data = array('success' => false, 'message' => $e->getMessage());
			return $response->withJson($data,200);
	    }		   
	
}





/**
 * Function Name: getScores
 *
 * Retrieves all scores 
 * 
 * @return (object) ($token) the actual scores details 
 */
function getScores($request, $response, $args) {	

	$sql = "SELECT id,user_id,score FROM scores";

	try {
		$db = getConnection();
		$stmt = $db->prepare($sql);	
		$stmt->execute();				  
		$scores = $stmt->fetchAll(PDO::FETCH_OBJ); 	 
		$db = null;
		
		$data = array('success' => true, 'scores' => $scores);
		return $response->withJson($data,200);
		
	} catch(PDOException $e) {		
		
		$data = array('success' => false, 
					  'message' => $e->getMessage(), 
					  'code' => $e->getCode(),
					  'one_line' => $e->getLine());

		return $response->withJson($data,200);

	}
}	




/**
 * Function Name: getScore
 *
 * Retrieves a score by id
 * 
 * @return (object) ($token) the actual score details 
 */
function getScore($request, $response, $args) {	

	$id = $args['id'];

	$sql = "SELECT id,user_id,score FROM scores WHERE id =:id";

	try {
		$db = getConnection();
		$stmt = $db->prepare($sql);	
		$stmt->bindParam("id", $id);	
		$stmt->execute();				  
		$score = $stmt->fetchObject();	 
		$db = null;
		
		$data = array('success' => true, 'score' => $score);
		return $response->withJson($data,200);
		
	} catch(PDOException $e) {		
		
		$data = array('success' => false, 
					  'message' => $e->getMessage(), 
					  'code' => $e->getCode(),
					  'one_line' => $e->getLine());

		return $response->withJson($data,200);

	}
}	




/**
 * Function Name: addScore
 *
 * Adds a new score
 *
 * @param (array) ([score details]) the score details
 * @return (number) ($rows) the number of rows added
 *
 */
function addScore($request, $response, $args) {	
	
	$detail = json_decode($request->getBody());
	
	$sql = "INSERT INTO scores (user_id,score) VALUES (:user_id, :score)";
	    
	    try {
	    	
	    	$score = cleanField($detail->score);
	    	$user_id = cleanField($detail->user_id);

	    	$score = intval($score);
	    		    			    	
	    	$db = getConnection();
			$stmt = $db->prepare($sql);
			
			$stmt->bindParam("score", $score);
			$stmt->bindParam("user_id", $user_id);		
			
			$rows = $stmt->execute();					

			$db = null;

			if($rows > 0){

				if($score < 80){

					$message = 0;
					$data = array('success' => true, 'message' => $message);
					return $response->withJson($data,200);

				}else{

					$array = checkIfInGift($user_id);

					$length = count($array);
					
					if($length > 0){ //user exists in the gifts table

						//save to drawers table
						$done = saveToDrawers($user_id, $score);

						if($done > 0){

							$message = 2;
							$data = array('success' => true, 'message' => $message);
							return $response->withJson($data,200);
						}

					}else{ //user does not exist in the gifts table

						$message = 1;
						$bottles = getBottleNumberInner();
						$data = array('success' => true, 'message' => $message, 'bottles' => $bottles);
						return $response->withJson($data,200);
					}

				}
									    
			    
			}else{
		    	
		    	$data = array('success' => false, 'message' => 'Sorry unable to add score');
				return $response->withJson($data,200);
		    }	    
		    
	    
	    } catch(PDOException $e) {		
		     
		    $data = array('success' => false, 'message' => $e->getMessage());
			return $response->withJson($data,200);
	    }		   
	
}






/**
 * Function Name: saveToDrawers
 *
 * Adds a new draw
 *
 * @param (array) ([draw details]) the draw details
 * @return (number) ($rows) the number of rows added
 *
 */
function saveToDrawers($user_id, $score) {	
	
	
	
	$sql = "INSERT INTO draws (user_id,score) VALUES (:user_id, :score)";
	    
	    try {
	    		    			    	
	    	$db = getConnection();
			$stmt = $db->prepare($sql);
			
			$stmt->bindParam("score", $score);
			$stmt->bindParam("user_id", $user_id);		
			
			$rows = $stmt->execute();					

			$db = null;

			return $rows;
	    
	    } catch(PDOException $e) {		
		     
		    $data = array('success' => false, 'message' => $e->getMessage());
			return $response->withJson($data,200);
	    }		   
	
}



/**
 * Function Name: checkIfInGift
 *
 * Checks if a user ID exists in the gifts table
 *
 * @param (array) ([draw details]) the draw details
 * @return (number) ($rows) the number of rows added
 *
 */
function checkIfInGift($user_id) {	
	
	$sql = "SELECT id,user_id,gift FROM gifts WHERE user_id =:user_id";

	try {
		$db = getConnection();
		$stmt = $db->prepare($sql);
		$stmt->bindParam("user_id", $user_id);	
		$stmt->execute();				  
		$gifts = $stmt->fetchAll(PDO::FETCH_OBJ); 	 
		$db = null;
		
		return $gifts;
		
	} catch(PDOException $e) {		
		
		$data = array('success' => false, 
					  'message' => $e->getMessage(), 
					  'code' => $e->getCode(),
					  'one_line' => $e->getLine());

		return $response->withJson($data,200);

	}

}





/**
 * Function Name: getBottleNumber
 *
 * Retrieves total number of bottles awarded today 
 * 
 * @return (object) ($token) the actual bottles number
 */
function getBottleNumberInner() {	

	$sql = "SELECT count(*) as total FROM gifts WHERE gift =:gift AND DATE(created_at) = DATE(NOW())";
	
	$drink = 'drink';

	try {
		
		$db = getConnection();
		$stmt = $db->prepare($sql);	
		$stmt->bindParam("gift", $drink);	
		$stmt->execute();				  
		$number = $stmt->fetchObject();	 
		$db = null;		
		
		return $number;
		
	} catch(PDOException $e) {		
		
		$data = array('success' => false, 
					  'message' => $e->getMessage(), 
					  'code' => $e->getCode(),
					  'one_line' => $e->getLine());

		return $response->withJson($data,200);

	}
}	





/**
 * Function Name: getAllGifts
 *
 * Retrieves all gifts 
 * 
 * @return (object) ($token) the actual gifts details 
 */
function getAllGifts($request, $response, $args) {	

	$sql = "SELECT g.user_id, gift, date_format(g.created_at, '%d-%M-%Y') as created_at, name, mobile_number, department FROM gifts as g INNER JOIN users as u ON u.id = g.user_id";

	try {
		$db = getConnection();
		$stmt = $db->prepare($sql);	
		$stmt->execute();				  
		$gifts = $stmt->fetchAll(PDO::FETCH_OBJ); 	 
		$db = null;	

		ob_clean();

		$data = array('success' => true, 'gifts' => $gifts);
		
		$body = $response->getBody();
		$body->write(json_encode($data, JSON_UNESCAPED_SLASHES));
		$k = $body->getSize();
		echo '   ';
		
	} catch(PDOException $e) {		
		
		$data = array('success' => false, 
					  'message' => $e->getMessage(), 
					  'code' => $e->getCode(),
					  'one_line' => $e->getLine());

		return $response->withJson($data,200);

	}
}	






/**
 * Function Name: getAllScores
 *
 * Retrieves all scores 
 * 
 * @return (object) ($token) the actual scores details 
 */
function getAllScores($request, $response, $args) {	

	$sql = "SELECT user_id, score, date_format(s.created_at, '%d-%M-%Y') as created_at, name, mobile_number, department FROM scores as s INNER JOIN users as u ON u.id = s.user_id";

	try {
		$db = getConnection();
		$stmt = $db->prepare($sql);	
		$stmt->execute();				  
		$scores = $stmt->fetchAll(PDO::FETCH_OBJ); 	 
		$db = null;	

		ob_clean();

		$data = array('success' => true, 'scores' => $scores);
		
		$body = $response->getBody();
		$body->write(json_encode($data, JSON_UNESCAPED_SLASHES));
		$k = $body->getSize();
		echo '   ';
		
	} catch(PDOException $e) {		
		
		$data = array('success' => false, 
					  'message' => $e->getMessage(), 
					  'code' => $e->getCode(),
					  'one_line' => $e->getLine());

		return $response->withJson($data,200);

	}
}	






//Check if the email address is valid
function checkEmail($str){
	return preg_match("/^[\.A-z0-9_\-\+]+[@][A-z0-9_\-]+([.][A-z0-9_\-]+)+[A-z]{1,4}$/", $str);
}


//Check for illegal characters
function cleanField($text){
	
	$name = preg_replace("/[^a-zA-Z0-9\/_|+,<>\[\]() .-]/", '', $text);//remove special characters
	$name = preg_replace("/[\"\']/", '', $name); //remove any quotes

	return $name;
}





?>
